

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_System_Chassis' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_System_Chassis]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE FUNCTION [dbo].[f_System_Chassis] ()
  RETURNS TABLE
AS

RETURN
  SELECT
      c.ComputerID                     AS ComputerID,
      MAX(c.Value)                     AS TypeID,
      MAX(CASE c.Value
            WHEN '1' then 'Other'
            WHEN '2' then 'Unknown'
            WHEN '3' then 'Desktop'
            WHEN '4' then 'Low Profile Desktop'
            WHEN '5' then 'Pizza Box'
            WHEN '6' then 'Mini Tower'
            WHEN '7' then 'Tower'
            WHEN '8' then 'Portable'
            WHEN '9' then 'Laptop'
            WHEN '10' then 'Notebook'
            WHEN '11' then 'Hand Held'
            WHEN '12' then 'Docking Station'
            WHEN '13' then 'All in One'
            WHEN '14' then 'Sub Notebook'
            WHEN '15' then 'Space-Saving'
            WHEN '16' then 'Lunch Box'
            WHEN '17' then 'Main System Chassis'
            WHEN '18' then 'Expansion Chassis'
            WHEN '19' then 'SubChassis'
            WHEN '20' then 'Bus Expansion Chassis'
            WHEN '21' then 'Peripheral Chassis'
            WHEN '22' then 'Storage Chassis'
            WHEN '23' then 'Rack Mount Chassis'
            WHEN '24' then 'Sealed-Case PC'
            ELSE 'Unknown'
          END)                         AS Type
    FROM (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties('CIM_Chassis') WHERE Property = 'ChassisTypes') c
    GROUP BY c.ComputerID

GO
	

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_System_Processor' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_System_Processor]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE FUNCTION [dbo].[f_System_Processor] ()
  RETURNS TABLE
AS

RETURN
  SELECT
      p.ComputerID                               AS ComputerID,
      MAX(COALESCE(p.Value, descr.Value))        AS Name,
      CASE
        WHEN MAX(model.Value) LIKE '%Intel%' THEN 'Intel'
        WHEN MAX(model.Value) LIKE '%AMD%'   THEN 'AMD'
        WHEN MAX(descr.Value) LIKE '%Intel%' THEN 'Intel'
        WHEN MAX(descr.Value) LIKE '%AMD%'   THEN 'AMD'
        WHEN MAX(p.Value) LIKE '%PowerPC%'   THEN 'IBM'
        WHEN MAX(p.Value) LIKE '%SPARC%'     THEN 'SUN/Oracle'
        ELSE MAX(model.Value)
      END                                        AS Manufacturer,
      ROUND(CONVERT(REAL, MAX(sp.Value))/1024,2) AS Speed,
/*    COUNT(*)                                   AS CPUs,
      MAX(core.Value)                            AS Cores,
      MAX(logical.Value)                         AS Logical, */
      CASE
        WHEN MAX(lVMCapacity.Value) IS NOT NULL AND MAX(p.Value) LIKE '%SPARC%' THEN CAST(ROUND(MAX(lVMCapacity.Value),0) AS INT)
        WHEN MAX(unixCPUs.Value) IS NOT NULL AND MAX(p.Value) LIKE '%PowerPC%' THEN CAST(ROUND(MAX(unixCPUs.Value),0) AS INT)
        ELSE COUNT(*)
      END                                        AS CPUs,
      CASE
        WHEN MAX(lVMCapacityInCores.Value) IS NOT NULL AND MAX(p.Value) LIKE '%SPARC%' THEN CAST(ROUND(MAX(lVMCapacityInCores.Value),0) AS INT)
        WHEN MAX(unixCPUs.Value) IS NOT NULL AND MAX(p.Value) LIKE '%PowerPC%' THEN NULL
        ELSE MAX(core.Value)
      END                                        AS Cores,
      CASE
        WHEN MAX(lVMCapacityInCores.Value) IS NOT NULL AND MAX(p.Value) LIKE '%SPARC%' THEN NULL
        WHEN MAX(unixLogical.Value) IS NOT NULL AND MAX(p.Value) LIKE '%PowerPC%' THEN CAST(ROUND(MAX(unixLogical.Value),0) AS INT)
        ELSE MAX(logical.Value)
      END                                        AS Logical, 
      CASE
        WHEN MAX(model.Value) LIKE '%Intel%' AND CAST(MAX(logical.Value) AS INT) > CAST(MAX(core.Value) AS INT) THEN 1
        WHEN MAX(model.Value) LIKE '%AMD%' AND CAST(MAX(logical.Value) AS INT) > CAST(MAX(core.Value) AS INT) THEN 1
        WHEN MAX(descr.Value) LIKE '%Intel%' AND CAST(MAX(logical.Value) AS INT) > CAST(MAX(core.Value) AS INT) THEN 1
        WHEN MAX(descr.Value) LIKE '%AMD%' AND CAST(MAX(logical.Value) AS INT) > CAST(MAX(core.Value) AS INT) THEN 1
        ELSE 0
      END                                        AS HyperThreating
    FROM (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties('CIM_Processor') WHERE Property = 'Name') p
      LEFT JOIN f_HardwareValuesByPropertyName('Manufacturer') model
        ON p.HardwareObjectID = model.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('Description') descr
        ON p.HardwareObjectID = descr.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('MaxClockSpeed') sp
        ON p.HardwareObjectID = sp.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('NumberOfCores') core
        ON p.HardwareObjectID = core.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('NumberOfLogicalProcessors') logical
        ON p.HardwareObjectID = logical.HardwareObjectID
      LEFT JOIN (SELECT ComputerID, HardwareObjectID, Value 
                   FROM f_HardwareProperties('CIM_UnitaryComputerSystem') 
                   WHERE Property = 'VMCapacity') lVMCapacity
        ON p.ComputerID = lVMCapacity.ComputerID
      LEFT JOIN f_HardwareValuesByPropertyName('VMCapacityInCores') lVMCapacityInCores
        ON lVMCapacity.HardwareObjectID = lVMCapacityInCores.HardwareObjectID
      LEFT JOIN (SELECT ComputerID, HardwareObjectID, Value 
                   FROM f_HardwareProperties('CIM_UnitaryComputerSystem') 
                   WHERE Property = 'NumberOfProcessors') unixCPUs
        ON p.ComputerID = unixCPUs.ComputerID
      LEFT JOIN f_HardwareValuesByPropertyName('NumberOfLogicalProcessors') unixLogical
        ON unixCPUs.HardwareObjectID = unixLogical.HardwareObjectID
    GROUP BY p.ComputerID
GO


IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_System_MemoryDisk' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_System_MemoryDisk]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 22.01.2018
-- Last change: 22.01.2018 First Set
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Total Memory and Total Disk Space
-- ===============================================
CREATE FUNCTION [dbo].[f_System_MemoryDisk] ()
  RETURNS TABLE
AS

RETURN
  SELECT
      m.ComputerID,
      m.Value        AS MemoryTotal,
      d.Value        AS DiskTotal
    FROM (SELECT
              ComputerID,
              ROUND(CAST(Value AS BIGINT)/1024/1024/1024,2) AS Value 
            FROM f_HardwareProperties('CIM_UnitaryComputerSystem')
            WHERE Property = 'TotalPhysicalMemory'
              AND Value NOT LIKE '%[^0-9]%' AND LEN(Value) < 19) m
      LEFT JOIN (SELECT
                      ComputerID, 
                      ROUND(SUM(CAST(Value AS BIGINT)/1024/1024/1024),2) AS Value 
                    FROM f_HardwareProperties('CIM_DiskDrive')
                    WHERE Property = 'Size'
                      AND Value NOT LIKE '%[^0-9]%' AND LEN(Value) < 19
                    GROUP BY ComputerID) d
        ON m.ComputerID = d.ComputerID
    WHERE m.Value IS NOT NULL OR d.Value IS NOT NULL

GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_HardwareValuesByPropertyName' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_HardwareValuesByPropertyName]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

CREATE FUNCTION [dbo].[f_HardwareValuesByPropertyName] (@PropertyName NVARCHAR(255))
      RETURNS TABLE
AS

RETURN
  SELECT HardwareObjectID, Value
    FROM HardwareValue AS hv
      INNER JOIN HardwareProperty AS hp
        ON hp.HardwarePropertyID = hv.HardwarePropertyID
    WHERE Property = @PropertyName
	
GO

IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_Virtual_HyperV_HostInfo' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_Virtual_HyperV_HostInfo]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO
-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 16.03.2018
-- Last change: 
-- Copyright:   (c) Raynet GmbH
-- Description:	Get all Information from all HyperV Hosts
--              ndtrack and RWI
-- ===============================================
-- Used Functions:
-- - for All
--   - f_HardwareProperties(<ClassName>)
--   - f_HardwareValuesByPropertyName(<PropertyName>)
CREATE FUNCTION [dbo].[f_Virtual_HyperV_HostInfo]()
  RETURNS TABLE
AS

RETURN
-- HyperV from RWI Data
  SELECT
      gN.ComputerID                        AS ComputerID,
      gN.Value                             AS SystemGuid,
      gE.Value                             AS ElementName,
	  gD.Value                             AS Discription,
	  CAST(NULL AS NVARCHAR(255))          AS DNSNameGuest,
      gDa.Value                            AS InstallDate,
      CASE gs.Value
        WHEN '0' THEN 'Unknown'
        WHEN '2' THEN 'Running'
        WHEN '3' THEN 'Off'
        WHEN '6' THEN 'Saved'
        WHEN '32768' THEN 'Paused'
        WHEN '32769' THEN 'Suspended'
        WHEN '32770' THEN 'Starting'
        WHEN '32771' THEN 'Snapshotting'
        WHEN '32773' THEN 'Saving'
        WHEN '32774' THEN 'Stopping'
        WHEN '32775' THEN 'Pausing'
        WHEN '32775' THEN 'Resuming'
        ELSE 'Unknown'
      END                                  AS [State],
	  gs.Value                             AS StateInt,
      bG.Value                             AS BIOSGuid,
      bs.Value                             AS BIOSSerialNumber,
	  mVQ.Value                            AS Memory,
      mMin.Value                           AS MemoryMin,
      mMax.Value                           AS MemoryMax,
      pVQ.Value                            AS GuestProcessorCount
    FROM (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties('Msvm_ComputerSystem') WHERE Property = 'Name') gN
      LEFT JOIN f_HardwareValuesByPropertyName('ElementName') gE
        ON gN.HardwareObjectID = gE.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('Description') gD
        ON gN.HardwareObjectID = gD.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('InstallDate') gDa
        ON gN.HardwareObjectID = gDa.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('EnabledState') gS
        ON gN.HardwareObjectID = gS.HardwareObjectID
      LEFT JOIN f_HardwareProperties('Msvm_VirtualSystemSettingData') bId
        ON gN.ComputerID = bId.ComputerID
          AND bId.Property = 'INSTANCEID'
          AND bId.Value = 'Microsoft:' + gN.Value
      LEFT JOIN f_HardwareValuesByPropertyName('BIOSGUID') bG
        ON bId.HardwareObjectID = bG.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('BIOSSerialNumber') bS
        ON bId.HardwareObjectID = bS.HardwareObjectID
      LEFT JOIN f_HardwareProperties('Msvm_MemorySettingData') mId
        ON gN.ComputerID = mId.ComputerID
          AND mId.Property = 'INSTANCEID'
          AND mId.Value LIKE 'Microsoft:' + gN.Value + '%'
      LEFT JOIN f_HardwareValuesByPropertyName('VirtualQuantity') mVQ
        ON mId.HardwareObjectID = mVQ.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('Limit') mMax
        ON mId.HardwareObjectID = mMax.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('Reservation') mMin
        ON mId.HardwareObjectID = mMin.HardwareObjectID
      LEFT JOIN f_HardwareProperties('Msvm_ProcessorSettingData') pId
        ON gN.ComputerID = pId.ComputerID
          AND pId.Property = 'InstanceID'
          AND pId.Value LIKE 'Microsoft:' + gN.Value + '%'
      LEFT JOIN f_HardwareValuesByPropertyName('VirtualQuantity') pVQ
        ON pId.HardwareObjectID = pVQ.HardwareObjectID
/*      LEFT JOIN f_HardwareProperties('Msvm_KvpExchangeComponent') dId
        ON gN.ComputerID = dId.ComputerID
          AND dId.Property = 'Systemname'
          AND dId.Value = gN.Value
      LEFT JOIN f_HardwareValuesByPropertyName('FullyQualifiedDomainName') dFQDN
        ON dId.HardwareObjectID = dFQDN.HardwareObjectID

		Set arrKvpComponents = oWMIVM.ExecQuery("SELECT * FROM Msvm_KvpExchangeComponent WHERE Systemname='" & oGuest.Name & "'") '.ItemIndex(0)
		sDNSName = ""
		For Each oKvpComponent in arrKvpComponents
			For Each oKvpItem In oKvpComponent.GuestIntrinsicExchangeItems
				oXML.loadXML(oKvpItem)
				call oXML.setProperty("SelectionLanguage", "XPath")
				set oSelect = oXML.SelectSingleNode("/INSTANCE/PROPERTY[@NAME='Name']/VALUE")
				If not IsNull(oSelect) and not IsEmpty(oSelect) and oSelect.Text = "FullyQualifiedDomainName" Then
					set oValue = oXML.SelectSingleNode("/INSTANCE/PROPERTY[@NAME='Data']/VALUE")
					sDNSName = oValue.Text
				End If
			Next
		Next
*/
    WHERE gD.Value != 'Microsoft Hosting Computer System'
-- HyperV from ndtrack PlugIN
  UNION SELECT
      gN.ComputerID                        AS ComputerID,
      gN.Value                             AS SystemGuid,
      gE.Value                             AS ElementName,
	  gD.Value                             AS Discription,
	  dFQDN.Value                          AS DNSNameGuest,
      gDa.Value                            AS InstallDate,
      gs.Value                             AS [State],
	  gsi.Value                            AS StateInt,
      bG.Value                             AS BIOSGuid,
      bs.Value                             AS BIOSSerialNumber,
	  mVQ.Value                            AS Memory,
      mMin.Value                           AS MemoryMin,
      mMax.Value                           AS MemoryMax,
      pVQ.Value                            AS GuestProcessorCount
    FROM (SELECT ComputerID, HardwareObjectId, Value FROM f_HardwareProperties('HyperV_Guest') WHERE Property = 'SystemGuid') gN
      LEFT JOIN f_HardwareValuesByPropertyName('ElementName') gE
        ON gN.HardwareObjectID = gE.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('Description') gD
        ON gN.HardwareObjectID = gD.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('InstallDate') gDa
        ON gN.HardwareObjectID = gDa.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('State') gS
        ON gN.HardwareObjectID = gS.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('StateInt') gSi
        ON gN.HardwareObjectID = gSi.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('BIOSGuid') bG
        ON gN.HardwareObjectID = bG.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('BIOSSerialNumber') bS
        ON gN.HardwareObjectID = bS.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('Memory') mVQ
        ON gN.HardwareObjectID = mVQ.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('MemoryMax') mMax
        ON gN.HardwareObjectID = mMax.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('MemoryMin') mMin
        ON gN.HardwareObjectID = mMin.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('ProcessorCount') pVQ
        ON gN.HardwareObjectID = pVQ.HardwareObjectID
      LEFT JOIN f_HardwareValuesByPropertyName('DNSNameGuest') dFQDN
        ON gN.HardwareObjectID = dFQDN.HardwareObjectID		
GO		


IF EXISTS (
        SELECT * FROM dbo.sysobjects
        WHERE [name] = 'f_System_Base' AND xtype = 'IF'
)
        DROP FUNCTION [dbo].[f_System_Base]
GO
SET ANSI_NULLS ON
GO
SET QUOTED_IDENTIFIER ON
GO

-- ===============================================
-- Author:      P. Arbogast, Raynet GmbH
-- Create date: 11.2016
-- Last change: 12.2016 - Remove Memory and Disk, make Problem on some Systems
--              04.06.2017 fix multiple lines per device
--              01.12.2017 fix multiple lines per device
--              20.12.2017 Fix for ComputerCN with IPAddress and Inventories
--              26.01.2018 fix for ComputerCN change from IPAddress Filter to take always opCSName if not null
--              26.01.2018 fix for Memory and Diskspace
--              02.03.2018 fix Manufacturer and Model for Fujitsu, IBM Server
--              02.03.2018 add ad Model Information the Amazon Cloud Version
--              02.03.2018 fix FQDN
--              06.04.2018 fix Double Entrys
--              16.05.2018 fix OS Name
--                         fix Model if no Manufacturer exist
--                         add Manufacture by System Enclosure
-- Copyright:   (c) Raynet GmbH
-- Description:	Get Base Hardwareinformations from All Systems
-- ===============================================
CREATE FUNCTION [dbo].[f_System_Base] ()
  RETURNS TABLE
AS

RETURN
  SELECT
        c.ComputerID                                          AS ComputerID,
-- 20.12.2017 Fix for ComputerCN with IPAddress and Inventories
        CASE
          WHEN opCSName.Value IS NOT NULL THEN opCSName.Value
          ELSE c.ComputerCN
        END                                                   AS ComputerName,
        fqdn.FQDN                                             AS ComputerFQDN,
-- 02.03.2018 Fix Manuifacturer and Model for Fujitsu and IBM Server
        COALESCE(base.Manufacturer, basePro.Manufacturer, man.Value) AS Manufacturer,
        COALESCE(basePro.Model, base.Model)                   AS Model,
        chassis.Type                                          AS ChassisType,
        serial.Number                                         AS SerialNumber,
        biosVer.Value                                         AS BiosVersion,
        COALESCE(osName.Name, osName2.Value) AS OS,
        processor.Manufacturer                                AS CPUManufacturer,
        processor.Name                                        AS CPUModel,
        processor.CPUs                                        AS CPUs,
        processor.Cores                                       AS Cores,
        processor.HyperThreating                              AS HyperThreating,
        processor.Speed                                       AS CPUSpeed,
        memDisk.MemoryTotal                                   AS Memory,
        memDisk.DiskTotal                                     AS DiskSize,
        lLogin.Value                                          AS LastLogin,
        ir.HWDate                                             AS InventoryDate
    FROM Computer c WITH (NOLOCK)
  -- Memory / Disk Size
-- 26.01.2017 fix for Memory and Diskspace
        LEFT JOIN f_System_MemoryDisk() memDisk
          ON c.ComputerID = memDisk.ComputerID
  -- DNS
       LEFT JOIN (SELECT DISTINCT
                      c.ComputerID,
                      CASE
                        WHEN CHARINDEX('.', c.ComputerCN) > 0 THEN c.ComputerCN
                        WHEN dns.DNSSuffix IS NOT NULL THEN c.ComputerCN + '.' + dns.DNSSuffix
                        WHEN dbo.CIMString(nd.DNSFullName) IS NOT NULL THEN nd.DNSFullName
                        WHEN CHARINDEX(',', d.DN) > 1 THEN c.ComputerCN + REPLACE(ISNULL('.' + REPLACE(REPLACE(dbo.CIMString(o.RDN), 'CN=', ''), 'OU=', ''), '') + '.' + REPLACE(d.DN, 'DC=', ''), ',', '.')
--                        WHEN CHARINDEX('.', baseDom.Value) > 0 THEN baseDom.Value
                        ELSE c.ComputerCN
                      END FQDN
                   FROM Computer c WITH (NOLOCK)
                     LEFT JOIN Organization o WITH (NOLOCK)
                       ON c.ComputerOUID = o.OrganizationID
                     LEFT JOIN Domain d WITH (NOLOCK)
					       ON o.DomainID = d.OrganizationID
                     LEFT JOIN NetworkDevice nd WITH (NOLOCK)
-- 01.12.2017 add filter on DupplicateID IS NULL, only the last one is from intresting
-- 01.12.2017 add filter on computerCN to DeviceName, some times it give entrys with other devicename what
--            is mapped to the ComputerDevice, this are not from intresst
                       ON nd.DuplicateID IS NULL AND c.ComputerID = nd.ComputerID AND c.ComputerCN = nd.DeviceName 
                     LEFT JOIN f_HardwareProperties('CIM_UnitaryComputerSystem') baseDom
                       ON c.ComputerID = baseDom.ComputerID
                         AND Property = 'Domain'
                     LEFT JOIN (SELECT
                                     nic.ComputerID,
                                     MAX(dns.Value) AS DNSSuffix
                                   FROM f_HardwareProperties('Win32_NetworkAdapterConfiguration') nic
                                     LEFT JOIN f_HardwareValuesByPropertyName('DNSDomain') dns
                                       ON nic.HardwareObjectID = dns.HardwareObjectID
                                   WHERE nic.Property = 'IPEnabled' AND nic.Value = 'TRUE' AND dns.Value IS NOT NULL
                                   GROUP BY nic.ComputerID) dns
                       ON c.ComputerID = dns.ComputerID
                 ) fqdn
          ON c.ComputerID = fqdn.ComputerID
  -- Operating System
        LEFT JOIN (SELECT
                       osN.ComputerID, 
                       osN.HardwareObjectID, 
                       osN.Value + ' ' + osVersion.Value AS Name 
                     FROM f_HardwareProperties('CIM_OperatingSystem') osN
                       LEFT JOIN f_HardwareValuesByPropertyName('Version') osVersion
                         ON osN.HardwareObjectID = osVersion.HardwareObjectID
                     WHERE osN.Property = 'Name' AND osN.Value IS NOT NULL) osName
          ON c.ComputerID = osName.ComputerID
-- 15.05.2018 Caption instant of Name
        LEFT JOIN f_HardwareProperties('CIM_OperatingSystem') osName2
          ON c.ComputerID = osName2.ComputerID
            AND osName2.Property = 'Caption' AND osName2.Value IS NOT NULL
-- 20.12.2017 Fix for ComputerCN with IPAddress and Inventories
        LEFT JOIN f_HardwareValuesByPropertyName('CSNAME') opCSName
          ON osName.HardwareObjectID = opCSName.HardwareObjectID
  -- Processor
        LEFT JOIN f_System_Processor() processor
          ON c.ComputerID = processor.ComputerID
  -- SerialNumber
        LEFT JOIN (SELECT
                       ser.ComputerID, 
                       ser.HardwareObjectID, 
                       MAX(ser.Value) AS Number
                     FROM (SELECT ComputerID, HardwareObjectID, Value, Property FROM f_HardwareProperties('CIM_BIOSElement') WHERE Property IN ('SerialNumber')) ser
                     GROUP BY ComputerID, ser.HardwareObjectID) serial
          ON c.ComputerID = serial.ComputerID
  -- Bios Version
        LEFT JOIN f_HardwareValuesByPropertyName('Version') biosVer
          ON serial.HardwareObjectID = biosVer.HardwareObjectID
  -- Manufacturer and Model
-- 02.03.2018 Fix Manufacturer and MOdel for Fujistu and IBM Server
        LEFT JOIN (SELECT
                       COALESCE(man.HardwareObjectID, model.HardwareObjectID)   AS HardwareObjectID,
                       COALESCE(man.ComputerID, model.ComputerID)               AS ComputerID,
                       man.Value                                                AS Manufacturer,
                       model.Value                                              AS Model
                     FROM (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties('CIM_UnitaryComputerSystem') WHERE Property = 'Manufacturer') man
-- 16.05.2018 Show Model Everytime
                       FULL OUTER JOIN (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties('CIM_UnitaryComputerSystem') WHERE Property = 'Model') model
                         ON man.HardwareObjectID = model.HardwareObjectID) base
          ON c.ComputerID = base.ComputerID
-- Added for the Fix Fujitsu IBM Server
        LEFT JOIN (SELECT
                       man.HardwareObjectID                                     AS HardwareObjectID,
                       man.ComputerID                                           AS ComputerID,
                       man.Value                                                AS Manufacturer,
                       CASE
                         WHEN LEN(prdVer.Value) < 5          THEN prdName.Value                           -- Some IBM Srever has in the Version realy a Version and then its the Name
                         WHEN man.Value IN ('LENOVO', 'IBM') THEN COALESCE(prdVer.Value, prdName.Value)   -- For IBM take this
                         WHEN prdName.Value = 'HVM domU'     THEN prdName.Value + ' ' + prdVer.Value      -- 02.03.2018 Add Version Information if Amazon Cloud
                         ELSE prdName.Value                                                               -- For the rest like Fujitsu
                       END                                                      AS Model
                     FROM (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties('CIM_Product') WHERE Property = 'Vendor') man
                       LEFT JOIN (SELECT ComputerID, HardwareObjectID, Value FROM f_HardwareProperties('CIM_Product') WHERE Property = 'Name') prdName
                         ON man.ComputerID = prdName.ComputerID
                       LEFT JOIN f_HardwareValuesByPropertyName('Version') prdVer
                         ON prdName.HardwareObjectID = prdVer.HardwareObjectID) basePro
          ON c.ComputerID = basePro.ComputerID
  -- Fix 16.05.2018 Manufacturer
        LEFT JOIN (SELECT ComputerID, MAX(Value) AS Value FROM f_HardwareProperties('CIM_Chassis') WHERE Property = 'Manufacturer' AND Value != 'No Enclosure' GROUP BY ComputerID) man
          ON c.ComputerID = man.ComputerID
  -- Chassis Type
        LEFT JOIN f_System_Chassis() chassis
          ON c.ComputerID = chassis.ComputerID
  -- Inventory Date
        LEFT JOIN InventoryReport ir WITH (NOLOCK)
          ON c.ComputerID = ir.ComputerID
            AND ir.HWDate IS NOT NULL
  -- Last Login User
        LEFT JOIN f_HardwareValuesByPropertyName('Username') lLogin
          ON base.HardwareObjectID = lLogin.HardwareObjectID
GO